@@ -10,6 +10,8 @@ from graphio.helper import chunks, create_single_index, create_composite_index
 from graphio import defaults
 from graphio.queries import nodes_create_unwind, nodes_merge_unwind, nodes_merge_unwind_preserve, nodes_merge_unwind_array_props, \
     nodes_merge_unwind_preserve_array_props
+from graphio.graph import run_query_return_results
+
 log = logging.getLogger(__name__)
 
 # dict with python types to casting functions in Cypher
@@ -259,7 +261,7 @@ class NodeSet:
         return nodeset
 
     @classmethod
-    def from_csv_json_set(cls, csv_file_path, json_file_path):
+    def from_csv_json_set(cls, csv_file_path, json_file_path, load_items:bool = False):
         """
         Read the default CSV/JSON file combination.
 
@@ -267,6 +269,7 @@ class NodeSet:
 
         :param csv_file_path: Path to the CSV file.
         :param json_file_path: Path to the JSON file.
+        :param load_items: Yield items from file (False, default) or load them to memory (True).
         :return: The NodeSet.
         """
         with open(json_file_path) as f:
@@ -289,7 +292,10 @@ class NodeSet:
         # NodeSet instance
         nodeset = cls(metadata['labels'], merge_keys=metadata[mergekeys_json_key])
 
-        nodeset.nodes = _yield_node(csv_file_path, property_map)
+        if load_items:
+            nodeset.nodes = _read_nodes(csv_file_path, property_map)
+        else:
+            nodeset.nodes = _yield_node(csv_file_path, property_map)
 
         return nodeset
 
@@ -321,7 +327,7 @@ class NodeSet:
         with open(path, 'wt') as f:
             json.dump(self.to_dict(), f, indent=4)
 
-    def create(self, graph, batch_size=None):
+    def create(self, graph, database:str = None, batch_size=None):
         """
         Create all nodes from NodeSet.
         """
@@ -333,9 +339,9 @@ class NodeSet:
         q = nodes_create_unwind(self.labels)
 
         for batch in chunks(self.nodes, size=batch_size):
-            graph.run(q, props=list(batch))
+            run_query_return_results(graph, q, database=database, props=list(batch))
 
-    def merge(self, graph, merge_properties=None, batch_size=None, preserve=None, append_props=None):
+    def merge(self, graph, merge_properties=None, batch_size=None, preserve=None, append_props=None, database=None):
         """
         Merge nodes from NodeSet on merge properties.
 
@@ -362,26 +368,25 @@ class NodeSet:
         if not self.preserve and not self.append_props:
             q = nodes_merge_unwind(self.labels, self.merge_keys)
             for batch in chunks(self.node_properties(), size=batch_size):
-                graph.run(q, props=list(batch))
+                run_query_return_results(graph, q, database=database, props=list(batch))
 
         elif self.preserve and not self.append_props:
             q = nodes_merge_unwind_preserve(self.labels, self.merge_keys, property_parameter='props')
             for batch in chunks(self.node_properties(), size=batch_size):
-                graph.run(q, props=list(batch), preserve=self.preserve)
+                run_query_return_results(graph, q, database=database, props=list(batch), preserve=self.preserve)
 
         elif not self.preserve and self.append_props:
             q = nodes_merge_unwind_array_props(self.labels, self.merge_keys, self.append_props,
                                                property_parameter='props')
             for batch in chunks(self.node_properties(), size=batch_size):
-                graph.run(q, props=list(batch), append_props=self.append_props)
+                run_query_return_results(graph, q, database=database, props=list(batch), append_props=self.append_props)
 
         elif self.preserve and self.append_props:
 
             q = nodes_merge_unwind_preserve_array_props(self.labels, self.merge_keys, self.append_props, self.preserve,
                                                         property_parameter='props')
-            print(q)
             for batch in chunks(self.node_properties(), size=batch_size):
-                graph.run(q, props=list(batch), append_props=self.append_props, preserve=self.preserve)
+                run_query_return_results(graph, q, database=database, props=list(batch), append_props=self.append_props, preserve=self.preserve)
 
     def node_properties(self):
         """
@@ -435,7 +440,7 @@ class NodeSet:
 
         return property_types
 
-    def create_index(self, graph):
+    def create_index(self, graph, database=None):
         """
         Create indices for all label/merge ky combinations as well as a composite index if multiple merge keys exist.
 
@@ -448,11 +453,47 @@ class NodeSet:
             for label in self.labels:
                 # create individual indexes
                 for prop in self.merge_keys:
-                    create_single_index(graph, label, prop)
+                    create_single_index(graph, label, prop, database)
 
                 # composite indexes
                 if len(self.merge_keys) > 1:
-                    create_composite_index(graph, label, self.merge_keys)
+                    create_composite_index(graph, label, self.merge_keys, database)
+
+
+def _read_nodes(csv_filepath, property_map):
+    """
+    Instead of recreating the entire RelationShip set in memory this function yields
+    one relationship at a time.
+
+    :param csv_filepath: Path to the CSV file.
+    :param property_map: Property map to rename properties.
+    :return: One node property dict per iteration.
+    """
+
+    if csv_filepath.endswith('.gz'):
+        csvfile = gzip.open(csv_filepath, 'rt')
+    else:
+        csvfile = open(csv_filepath, newline='')
+    lines = csvfile.readlines()
+    csvfile.close()
+
+    # get header line
+    header = lines[0].strip().split(',')
+    header = [x.replace('"', '') for x in header]
+
+    log.debug(f"Header: {header}")
+
+    if property_map:
+        log.debug(f"Replace header {header}")
+        header = [property_map[x] if x in property_map else x for x in header]
+        log.debug(f"With header {header}")
+
+    nodes = []
+    rdr = csv.DictReader(lines[1:], fieldnames=header)
+    for node in rdr:
+        nodes.append(node)
+
+    return nodes
 
 
 def _yield_node(csv_filepath, property_map):
